
/*
 * Removal of this header is illegal.
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2005 Amduus Information Works, Inc.  www.amduus.com
 * Copyright (c) 2005 Scott Auge
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      Inc. and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Id$" NO-UNDO.

/***********************************************************************/
/* This program is meant to run as an object.                          */
/* It is a simple program to demonstrate:                              */
/*   -- "Private" attributes available to it                           */
/*   -- "Public" methods available to other programs.                  */
/*   -- A good template for future objects with Constructor and        */
/*      Destructor routines.                                           */
/***********************************************************************/

{objmgr.i} /* May be deleted if not using object manager */

FUNCTION cGetAttr RETURNS CHARACTER (INPUT cAttrName AS CHARACTER) FORWARD.

/* ----------------------- Begin Attributes List ----------------------*/

/* Example attribute specific to the object */

/* Used for object management */
DEFINE VARIABLE cgObjName          as character NO-UNDO.

/* These are global to the object */

DEFINE VARIABLE cNumberAlphabet AS CHARACTER NO-UNDO.
DEFINE VARIABLE cLetterAlphabet AS CHARACTER NO-UNDO.
DEFINE VARIABLE cPunctuationAlphabet AS CHARACTER NO-UNDO.
  
ASSIGN cNumberAlphabet = "0123456789"
       cLetterAlphabet = "abcdefghijklmnopqrstuvwxyz"
       cPunctuationAlphabet = "!*_@#$%^&()-+=~~~"~{~}~/~|~\<>~,./?:;~'[]".


/* ------------------------- End Attributes List ----------------------*/

/* -------------------------- Begin Methods List ----------------------*/

/***********************************************************************/
/* This is the "destructor" for the routine.  It should be called be   */
/* fore deleting the handle to the instance of this object.            */
/***********************************************************************/

PROCEDURE Destroy:

END.

/***********************************************************************/
/* If other "constructors" are needed, they can be put in.  This one   */
/* is called Init.  Unlike C++, it will need to be run manually.       */
/* If you are using ObjMgr.i, it will be run automatically.            */
/* If you need to make other Inits, place them here and name beginning */
/* "Init" : InitByRowID or InitBySalesOrderNumber.                     */
/***********************************************************************/

PROCEDURE Init:

  DEFINE INPUT PARAMETER cName AS CHARACTER NO-UNDO.

  ASSIGN cgObjName = cName.

  RUN SetError IN THIS-PROCEDURE ("000", "Init").
  RUN SetAttr IN THIS-PROCEDURE ("MinLength", "8").
  RUN SetAttr IN THIS-PROCEDURE ("MinChar", "6").
  RUN SetAttr IN THIS-PROCEDURE ("MinNum", "1").
  RUN SetAttr IN THIS-PROCEDURE ("MinPunc", "1").

END.

/***********************************************************************/
/* All procedures need a way to describe their errors back to the cal- */
/* ler.                                                                */
/***********************************************************************/

PROCEDURE GetError:

  DEFINE OUTPUT PARAMETER cErrCode AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cErrMsg  AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cErrMethod AS CHARACTER NO-UNDO.

  ASSIGN
  cErrCode = cGetAttr("ErrCode")
  cErrMsg  = cGetAttr("ErrMsg")
  cErrMethod = cGetAttr("ErrMethod").

END. /* PROCEDURE GetError */

/***********************************************************************/
/* This is the central point where internal procedures can communicate */
/* their problems to the procedure as a whole.                         */
/* It's main purpose is to convert codes into human readable form in   */
/* cgObjErrMsg as well as populate the procedures globally available   */
/* vars.                                                               */
/***********************************************************************/

PROCEDURE SetError:

  DEFINE INPUT PARAMETER cErrorCode AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER cMethodName AS CHARACTER NO-UNDO.

  CASE cErrorCode:

    WHEN "000" THEN DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", "No Error").
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.

    WHEN "001" THEN DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", "No Such Attribute").
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.

    WHEN "100" THEN DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", "Minimum Length Not Achieved").
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.

    WHEN "101" THEN DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", "Minimum Letters Not Achieved").
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.

    WHEN "102" THEN DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", "Minimum Numbers Not Achieved").
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.

    WHEN "103" THEN DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", "Minimum Punctuation Not Achieved").
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.
    
    OTHERWISE DO:
      RUN SetAttr IN THIS-PROCEDURE ("ErrCode", cErrorCode).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMsg", ?).
      RUN SetAttr IN THIS-PROCEDURE ("ErrMethod", cMethodName).
    END.

  END. /* CASE */

END. /* PROCEDURE SetError */

/***********************************************************************/
/* These are methods to perform activities on the data controlled by   */
/* the object.                                                         */
/* Attributes are actually stored in a temp-table, that way we can add */
/* new ones easily, and not need to create more Set/Get routines for   */
/* for each attribute.  The bad news is, we need to cast to and from   */
/* character for the data to pass back and fourth.  If this is a pron- */
/* lem, then create some Set/Get routines for those values.            */
/***********************************************************************/

DEFINE TEMP-TABLE ttAttributes
  FIELD AttrName AS CHARACTER
  FIELD AttrValue AS CHARACTER
  INDEX key1 AttrName ASCENDING.

/***********************************************************************/
/* We don't make this a function, because we need to FORWARD declare   */
/* in the code using this object, and this name is going to be pret-   */
/* ty popular causing conflict with other objects with a GetAttr.      */
/***********************************************************************/

PROCEDURE GetAttr:

  DEFINE INPUT PARAMETER cName   AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cValue AS CHARACTER NO-UNDO.

  FIND ttAttributes NO-LOCK
  WHERE ttAttributes.AttrName = cName
  NO-ERROR.

  IF NOT AVAILABLE ttAttributes THEN DO:

    /*********************************************************/
    /* Note that sometimes it is OK for an attribute to be ? */
    /* so be sure to remember to check the error if the at-  */
    /* tribute wasn't found or really is ?.  Sometimes it    */
    /* it works out it does not matter, sometimes it does    */
    /* matter.                                               */
    /*********************************************************/

    RUN SetError IN THIS-PROCEDURE ("001", "GetAttr").
    ASSIGN cValue = ?.
    RETURN.

  END. /* IF NOT AVAILABLE ttAttributes */

  ASSIGN cValue = ttAttributes.AttrValue.

END.  /* PROCEDURE GetAttr */

/***********************************************************************/
/* However it IS useful to have a GetAttr function for use in THIS-    */
/* PROCEDURE within the internal procedures available.                 */
/***********************************************************************/

FUNCTION cGetAttr RETURNS CHARACTER (INPUT cAttrName AS CHARACTER):

  DEFINE VARIABLE cAttrValue AS CHARACTER NO-UNDO.

  RUN GetAttr IN THIS-PROCEDURE (INPUT cAttrName, OUTPUT cAttrValue).

  RETURN cAttrValue.

END. /* FUNCTION cGetAttr() */

/***********************************************************************/
/* If the attribute already exists, we overwrite, not error out...     */
/***********************************************************************/

PROCEDURE SetAttr:

  DEFINE INPUT PARAMETER cName   AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER cValue AS CHARACTER NO-UNDO.

  FIND ttAttributes NO-LOCK
  WHERE ttAttributes.AttrName = cName
  NO-ERROR.

  IF NOT AVAILABLE ttAttributes THEN CREATE ttAttributes.

  ASSIGN ttAttributes.AttrName = cName
         ttAttributes.AttrValue = cValue.

  /* Some attributes are dependent on other attributes, handle those */
  /* here.                                                           */

  RUN DependentAttr (cName, cValue).

END. /* PROCEDURE SetAttr */

/***********************************************************************/
/* Some attributes are dependent on the values of other attributes.    */
/* We keep them in sync with this code here.                           */
/***********************************************************************/

PROCEDURE DependentAttr:

  DEFINE INPUT PARAMETER cName   AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER cValue AS CHARACTER NO-UNDO.

END. /* PROCEDURE DependentAttr */

/***********************************************************************/
/* This is a way to quickly transfer record information into attri-    */
/* butes.  The attribute name is tablename_fieldname.                  */
/* Example Use:                                                        */
/* FIND FIRST Person NO-LOCK.                                          */
/* ASSIGN hBuffer = BUFFER Person:Handle.                              */
/* RUN Record2Attr (hBuffer).                                          */
/***********************************************************************/

PROCEDURE Record2Attr:

  DEFINE INPUT PARAMETER hBuffer AS HANDLE NO-UNDO.

  DEFINE VARIABLE hField     AS HANDLE NO-UNDO.
  DEFINE VARIABLE i          AS INTEGER NO-UNDO.

  DO i = 1 TO hBuffer:Num-Fields:

    ASSIGN hField = hBuffer:Buffer-Field(i).

    RUN SetAttr IN THIS-PROCEDURE (hBuffer:Name + "_" + hField:Name, hField:String-Value).

  END.

END.

/***********************************************************************/
/* Scan the attributes table for entries beginning with the table name */
/* and apply their values to the field named in the second part of the */
/* attribute name.                                                     */
/* Note this doesn't manage multiple buffers of the same name very     */
/* well.  If you need n records from the same table, name the buffers  */
/* seperately.                                                         */
/***********************************************************************/

PROCEDURE Attr2Record:

  DEFINE INPUT PARAMETER hBuffer AS HANDLE NO-UNDO.

  DEFINE VARIABLE hField     AS HANDLE NO-UNDO.
  DEFINE VARIABLE cFieldName AS CHARACTER NO-UNDO.

  FOR EACH ttAttributes NO-LOCK
  WHERE ttAttributes.AttrName BEGINS hBuffer:Name + "_":

    /*****************************************************************/
    /* Luckily, it appears that buffer-value types automatically...  */
    /* It will puke on bad data sent - such as text for an int, etc. */
    /*****************************************************************/

    ASSIGN hfield = hBuffer:Buffer-Field(ENTRY(2, ttAttributes.AttrName, "_"))
           hfield:Buffer-Value = ttAttributes.AttrValue.

  END. /* FOR EACH ttAttributes */

END. /* PROCEDURE Attr2Record */

/***********************************************************************/
/* Useful for debugging. Note we output to a file so we don't get any  */
/* wrong display device type errors when the r-code is shared between  */
/* different interfaces.                                               */
/***********************************************************************/

PROCEDURE AttrDebug:

  DEFINE INPUT PARAMETER cFileName AS CHARACTER NO-UNDO.

  OUTPUT TO VALUE(cFileName).

  FOR EACH ttAttributes NO-LOCK:

    PUT UNFORMATTED ttAttributes.AttrName "=" ttAttributes.AttrValue SKIP.

  END. /* FOR EACH ttAttributes */

  OUTPUT CLOSE.

END. /* PROCEDURE AttrDebug */

/***********************************************************************/
/* Clear out all attributes that have been set.  It is better to set   */
/* an attribute to ? and code for that; but sometimes one needs this.  */
/***********************************************************************/

PROCEDURE ClearAttr:

  FOR EACH ttAttributes:

    DELETE ttAttributes.

  END. /* FOR EACH ttAttributes */

END.  /* PROCEDURE ClearAttr */

/***********************************************************************/
/* This is a special set of routines that allow the object's user to   */
/* reach the tables directly.  Convenient for ASSIGNS and the like so  */
/* one is not continously calling a Set* routine for each and every    */
/* field.                                                              */
/* WARNING: This CAN BE ABUSED.                                        */
/* WARNING: Sets of Like records may be a problem.                     */
/*    ATTN: Buffers can go both ways.  This can be good.  This can be  */
/*          bad.                                                       */
/***********************************************************************/
/*
PROCEDURE GetBuffers:

  DEFINE PARAMETER BUFFER YourRecord FOR YourTable.

  BUFFER-COPY ObjBuffer YourRecord.

END. /* PROCEDURE GetBuffers */

PROCEDURE SetBuffers:

  DEFINE PARAMETER BUFFER YourRecord FOR YourTable.

  BUFFER-COPY YourRecord ObjBuffer.

END.
*/

/* ----------------- Routines to implement password validation --------*/

PROCEDURE ValidatePassword:

  DEFINE INPUT PARAMETER cPassword AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iCount AS INTEGER NO-UNDO.
  
  IF cPassword = ? THEN ASSIGN cPassword = cGetAttr("Password").

  RUN SetAttr IN THIS-PROCEDURE ("Password", cPassword).
  RUN SetError IN THIS-PROCEDURE ("000", "ValidatePassword").
  
  IF LENGTH(cPassword) < INT(cGetAttr("MinLength")) THEN DO:
    RUN SetError IN THIS-PROCEDURE ("100", "ValidatePassword").
    RETURN.
  END.
  
  RUN PerformCount IN THIS-PROCEDURE ("Character", INPUT cPassword, OUTPUT iCount).
  
  IF iCount < INT(cGetAttr("MinChar")) THEN DO:
    /* DISPLAY "iCount < " INT(cGetAttr("MinChar")) SKIP.*/
    RUN SetError IN THIS-PROCEDURE ("101", "ValidatePassword").
    RETURN.
  END.

  RUN PerformCount IN THIS-PROCEDURE ("Number", INPUT cPassword, OUTPUT iCount).
  
  IF iCount < INT(cGetAttr("MinNum")) THEN DO:
    RUN SetError IN THIS-PROCEDURE ("102", "ValidatePassword").
    RETURN.
  END.
  
  RUN PerformCount IN THIS-PROCEDURE ("Punctuation", INPUT cPassword, OUTPUT iCount).
  
  IF iCount < INT(cGetAttr("MinPunc")) THEN DO:
    RUN SetError IN THIS-PROCEDURE ("103", "ValidatePassword").
    RETURN.
  END.  

END. /* PROCEDURE Validate Password */

/******************************************************************************/
/* Note: This routine requires "-rand 2" on the client command line parameter */
/******************************************************************************/

PROCEDURE GeneratePassword:

  DEFINE OUTPUT PARAMETER cPassword AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iIter AS INTEGER NO-UNDO.
  DEFINE VARIABLE iMinLen AS INTEGER NO-UNDO.
  DEFINE VARIABLE iMinChar AS INTEGER NO-UNDO.
  DEFINE VARIABLE iMinNum AS INTEGER NO-UNDO.
  DEFINE VARIABLE iMinPunc AS INTEGER NO-UNDO.
  
  RUN SetError IN THIS-PROCEDURE ("000", "GeneratePassword").
  
  ASSIGN iMinLen = INT(cGetAttr("MinLength"))
         iMinChar = INT(cGetAttr("MinChar"))
         iMinNum = INT(cGetAttr("MinNum"))
         iMinPunc = INT(cGetAttr("MinPunc")).
  
  /* Choose minimum set of characters */
  
  DO iIter = 1 TO iMinChar:
    ASSIGN cPassword = cPassword + SUBSTRING(cLetterAlphabet, RANDOM(1, LENGTH(cLetterAlphabet)), 1). 
  END.
  
  /* Choose minimum set of numbers */
  
  DO iIter = 1 TO iMinNum:
    ASSIGN cPassword = cPassword + SUBSTRING(cNumberAlphabet, RANDOM(1, LENGTH(cNumberAlphabet)), 1). 
  END.

  /* Choose minimum set of punctuation marks */
  
  DO iIter = 1 TO iMinPunc:
    ASSIGN cPassword = cPassword + SUBSTRING(cPunctuationAlphabet, RANDOM(1, LENGTH(cPunctuationAlphabet)), 1). 
  END.
  
  /* Make sure we have the minimum length */
  
  DO iIter = LENGTH(cPassword) TO iMinLen:
    ASSIGN cPassword = cPassword + SUBSTRING(cLetterAlphabet, RANDOM(1, LENGTH(cLetterAlphabet)), 1)
           cPassword = cPassword + SUBSTRING(cNumberAlphabet, RANDOM(1, LENGTH(cNumberAlphabet)), 1). 
  END. /* DO iIter = 1 TO iMinLen */
  
  /* Move our assemblies from above around */
  
  RUN RandomizeString IN THIS-PROCEDURE (INPUT-OUTPUT cPassword).
  
  RUN SetAttr IN THIS-PROCEDURE ("Password", cPassword).

END. /* GeneratePassword */

/******************************************************************************/
/* Note: This routine requires "-rand 2" on the client command line parameter */
/* list to be most effective.                                                 */
/* Might also be qualified for a "tools" library persistent procedure.        */
/******************************************************************************/

PROCEDURE RandomizeString:
  
  DEFINE INPUT-OUTPUT PARAMETER cText AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE iLen AS INTEGER NO-UNDO.
  DEFINE VARIABLE iIter AS INTEGER NO-UNDO.
  DEFINE VARIABLE cTemp AS CHARACTER NO-UNDO.
  DEFINE VARIABLE iSwap1 AS INTEGER NO-UNDO.
  DEFINE VARIABLE iSwap2 AS INTEGER NO-UNDO.
  
  ASSIGN iLen = LENGTH(cText).
  
  DO iIter = 1 TO iLen:
    
    ASSIGN iSwap1 = RANDOM (1, iLen)
           iSwap2 = RANDOM (1, iLen)
           cTemp = SUBSTRING(cText, iSwap1, 1)
           SUBSTRING(cText, iSwap1, 1) = SUBSTRING(cText, iSwap2, 1)
           SUBSTRING(cText, iSwap2, 1) = cTemp.
    
  END. /* DO iIter = 1 TO iLen */
  
END. /* PROCEDURE RandomizeString */

PROCEDURE PerformCount:
  
  DEFINE INPUT PARAMETER cType AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER cText AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER iCount AS INTEGER NO-UNDO INIT 0.
  
  DEFINE VARIABLE iLen AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCurIter AS INTEGER NO-UNDO.
  
  /* Put the alphabets into CHAR to aid with UNICODING and foreign languages */
  
  ASSIGN iLen = LENGTH(cText).
  
  CASE cType:
    
    WHEN "Number" THEN DO iCurIter = 1 TO iLen:
      IF INDEX (cNumberAlphabet, SUBSTRING(cText, iCurIter, 1)) > 0 THEN
        ASSIGN iCount = iCount + 1.
    END.
    
    WHEN "Character" THEN DO iCurIter = 1 TO iLen:
      IF INDEX (cLetterAlphabet, SUBSTRING(cText, iCurIter, 1)) > 0 THEN
        ASSIGN iCount = iCount + 1.  
    END.
    
    WHEN "Punctuation" THEN DO iCurIter = 1 TO iLen:
      IF INDEX (cPunctuationAlphabet, SUBSTRING(cText, iCurIter, 1)) > 0 THEN
        ASSIGN iCount = iCount + 1.  
    END.
    
  END. /* CASE */
  
  /* display cType ctext format "x(20)" icount iLen. */
  
END. /* PROCEDURE PerformCount */


/* -------------------------- End Methods List ------------------------*/
